!=========================================================================
! Copyright (C) GemTalk Systems 1986-2013.  All Rights Reserved.
!
! $Id: Error.gs 31898 2013-11-18 22:48:01Z stever $
!
!=========================================================================

removeallmethods Error
removeallclassmethods Error

category: 'For Documentation Installation only'
classmethod: Error
installDocumentation

| doc txt |
doc := GsClassDocumentation newForClass: self.

txt := (GsDocText new) details:
'The class protocol describe the behavior of the global Error. The value 
of the standard global Error is a class object that conforms to this 
protocol. The class Error is explicitly specified to be subclassable in 
a standard conforming program. Conforming implementations must implement 
its behaviors in a non-fragile manner.

The signaled exceptions generated by this type of object conform to the 
protocol <Error>.

The instance protocol describes the behavior of instances of class Error. 
These are used to represent error conditions that prevent the normal 
continuation of processing. Actual error exceptions used by an application 
may be subclasses of this class.

As Error is explicitly specified to be subclassable, conforming 
implementations must implement its behavior in a non- fragile manner.'.
doc documentClassWith: txt.

self description: doc.
%

! ------------------- Class methods for Error
category: 'ANSI support'
classmethod: Error
handles: anExceptionSelector

	^(self == Error and: [anExceptionSelector isKindOf: SymbolDictionary]) or: [
		super handles: anExceptionSelector.
	].
%
category: 'ANSI support'
classmethod: Error
handlesCategory: anObject number: anInteger

	(anObject == GemStoneError and: [anInteger // 1000 == 6]) ifTrue: [^false].
	^super handlesCategory: anObject number: anInteger.
%
! ------------------- Instance methods for Error
category: 'ANSI - signaledException'
method: Error
defaultAction
	"The current computation is terminated. The cause of the error should be
	logged or reported to the user. If the program is operating in an interactive
	debugging environment the computation should be suspended and the 
	debugger activated."

	^super defaultAction.
%
category: 'ANSI Support'
method: Error
isHaltPauseOrBreak

	^self gsCategory = GemStoneError and: [
		(#(6001 6002 6004 6005 6006) includes: self gsNumber) or: [
			self gsNumber = 2318 and: [self gsArguments = #(#'halt' '#halt encountered')].
		].
	].	
%
category: 'ANSI - exceptionDescription'
method: Error
isResumable
	"Error exceptions by default are assumed to not be 'resumable.' 
	Subclasses may override this definition for situations where it is
	appropriate for an error to be resumable."

	^self gsCategory = GemStoneError and: [
		(self gsNumber // 1000 = 6 and: [self gsNumber ~= 6011]) or: [
			self gsNumber = 2318 and: [self gsArguments = #(#'halt' '#halt encountered')].
		].
	].
%
