!=========================================================================
! Copyright (C) GemTalk Systems 2005-2013.  All Rights Reserved.
! 
!  segmentorder_1xto22.gs - to be filed into Gemstone64 v1.x prior to
!	conversion of repository to Gemstone64 v2.2.x
!
!  fixes ordering of Segments in SystemRepository to agree with 64bit image
!
! $Id: segmentorder_1xto22.gs 31898 2013-11-18 22:48:01Z stever $
!
!=========================================================================

! must be logged in as SystemUser

classmethod: Segment
new
"temporary implementation for conversion"

| res |
res := super new .
res initialize .
^ res
%  
method: Segment
initialize
"temporary implementation for conversion"
  itsRepository := SystemRepository.
  itsOwner := System myUserProfile.
  ownerAuthorization := 2.
  worldAuthorization := 0.
%
method: UserProfile
defaultSegment
"temporary implementation for conversion"

^defaultSegment
%
method: UserProfile
defaultSegment: aSegment
"temporary implementation for conversion"

defaultSegment := aSegment
%

expectvalue %String
run
"check naming in Globals of the segments with reserved oops"
| report namedSegBlk lf |
report := String new .
lf := Character lf .
namedSegBlk := [:aName :objId | | aSeg namedSeg |
  aSeg := Object _objectForOop: objId .
  aSeg == nil ifTrue:[
    report addAll: aName ; addAll: '  object ';
      addAll: objId asString ; addAll:' does not exist'; add: lf .
  ] ifFalse:[
    namedSeg := Globals at: aName otherwise: nil .
    report addAll: aName .
    namedSeg == aSeg ifTrue:[
      report addAll:' name ok '; add: lf .
    ] ifFalse:[
      namedSeg == nil ifTrue:[
        report addAll:' not in Globals, added'; add: lf .
      ] ifFalse:[
        report addAll:' wrong in Globals, changed value'; add: lf .
      ].
      Globals at: aName put: aSeg
    ].
  ].
]. 
namedSegBlk value: #SystemSegment      value: 1627 .
namedSegBlk value: #DataCuratorSegment value: 1631 .
namedSegBlk value: #SecurityDataSegment value: 1843 .
^ report
%

expectvalue %String
run
| hasOldSegs getSegBlk report lf 
  sysSeg dcSeg tzSeg idxSeg scdSeg pubSeg gcSeg namelessSeg 
  reposSegs firstSegs  identicalSegs  idx nonNilFirstSegs |
report := String new .
lf := Character lf .
reposSegs := Array withAll:SystemRepository .
report addAll:'starting segment count ', reposSegs size asString ; add: lf .
hasOldSegs := reposSegs size >= 8.
getSegBlk := [:aName :idx | |seg |
  seg := Globals at: aName otherwise: nil .
  seg == nil ifTrue:[ 
    report addAll:'segment ', aName; addAll:' does not exist, creating'; add: lf .
    seg := Segment new .
    Globals at: aName put: seg .
  ]. 
  seg == sysSeg ifTrue:[ 
    report addAll:'segment ', aName; addAll:' was aliased to SystemSegment, '.
    hasOldSegs ifTrue: [ seg := reposSegs at: idx.].
    seg == sysSeg ifTrue:[ 
       report addAll:'creating new'; add: lf .
       seg := Segment new .
       ] ifFalse: [
       report addAll: 'found in repository at ', idx asString; add: lf.
       ].
    Globals at: aName put: seg .
  ].
  seg 
].
sysSeg := Globals at: #SystemSegment  .
dcSeg := Globals at:#DataCuratorSegment .
tzSeg := getSegBlk value:#GsTimeZoneSegment value: 3.
idxSeg := getSegBlk value:#GsIndexingSegment value: 4.
scdSeg := getSegBlk value:#SecurityDataSegment value: 5.
pubSeg := getSegBlk value:#PublishedSegment value: 6 .

"ordering of firstSegs must match 64bit v2.2 image"
firstSegs := #[ sysSeg, dcSeg, tzSeg, idxSeg, scdSeg, pubSeg]. "gcSeg, namelessSeg"

(AllUsers userWithId:'SystemUser') defaultSegment ~~ sysSeg ifTrue:
  [(AllUsers userWithId:'SystemUser') defaultSegment: sysSeg ]. 

(AllUsers userWithId:'DataCurator') defaultSegment ~~ dcSeg ifTrue:
  [(AllUsers userWithId:'DataCurator') defaultSegment: dcSeg ]. 

gcSeg := (AllUsers userWithId:'GcUser') defaultSegment .
(gcSeg isNil or: [firstSegs includesIdentical: gcSeg]) ifTrue:[
  gcSeg := Segment new .
  (AllUsers userWithId:'GcUser') defaultSegment: gcSeg. 
  report addAll:'created new segment for GcUser'; add: lf .
].
firstSegs addLast: gcSeg . 
namelessSeg := (AllUsers userWithId:'Nameless') defaultSegment .
(namelessSeg isNil or: [firstSegs includesIdentical: namelessSeg]) ifTrue:[
  namelessSeg := Segment new .
  (AllUsers userWithId:'Nameless') defaultSegment: namelessSeg. 
  report addAll:'created new segment for Nameless'; add: lf .
].
firstSegs addLast: namelessSeg .


identicalSegs := IdentitySet withAll: firstSegs .
nonNilFirstSegs := IdentityBag withAll: firstSegs .
identicalSegs size = nonNilFirstSegs size ifFalse:[
  nil error:'bad size,  ', identicalSegs size asString , ' ' , nonNilFirstSegs size asString.
].
SystemRepository _basicSize: 0 .
1 to: 2 do:[:j | |aSeg |
  aSeg := firstSegs at:j .
  (aSeg ~~ nil) ifTrue:[
    SystemRepository _basicAt:j put: aSeg .
    reposSegs removeIdentical: aSeg ifAbsent:[
      report addAll:'(firstSegs at: ', j asString , 
	  ') not in reposSegs, added at ', j asString ; add: lf .
    ] .
  ].
].
idx := 3 .
3 to: firstSegs size do:[:j | |aSeg |
  aSeg := firstSegs at: j .
  (aSeg ~~ nil and:[ aSeg ~~ dcSeg and:[aSeg ~~ sysSeg]]) ifTrue:[
    SystemRepository _basicAt: idx put: aSeg .
    reposSegs removeIdentical: aSeg ifAbsent:[
      report addAll:'(firstSegs at: ', j asString , 
	 ') not in reposSegs, added at ', idx asString; add: lf .
    ] .
    idx :=  idx + 1 .
  ].
].
1 to: reposSegs size do:[:k | |aSeg |
  aSeg := reposSegs at: k .
  aSeg ~~ nil ifTrue:[
    (identicalSegs includesIdentical: aSeg) ifFalse:[
      identicalSegs add: aSeg .
      SystemRepository _basicAt:idx put: aSeg .
      idx := idx + 1 
    ] ifTrue:[
      report addAll:'found a duplicate segment'; add: lf .
    ].
  ].
].
report addAll:'ending segment count ', SystemRepository size asString; add: lf .
^ report
%

level 1
expectvalue %Repository
obj SystemRepository
level 0

